#ifndef __VCOPCODEHOOK_H__
#define __VCOPCODEHOOK_H__

struct RunningScript
{
	// Thanks to CyQ and PatrickW for this info. (Taken from Spooky's speedo)
	void *pNext;
	void *pPrev;
	char strName[8];
	unsigned long dwScriptIP;
	unsigned long dwReturnStack[6];
	unsigned long dwStackPointer;
	unsigned long dwLocalVar[18];
	unsigned char bStartNewScript;
	unsigned char bJumpFlag;
	unsigned char bWorky;
	unsigned char bAwake;
	unsigned long dwWakeTime;
	unsigned short wIfParam;
	unsigned char bNotFlag;
	unsigned char bWastedBustedCheck;
	unsigned char bWastedBustedFlag;
	unsigned char bMissionThread;
	unsigned char pad[2];
};

struct IOpcodeHookMgr
{
	/* add:
		Adds a new opcode hook. If the specified opcode is already hooked then this
		function will fail. The handler function must return a bool (scm compare flag)
		and must be of __cdecl calling convention.

		Example scm "pseudo" (Barton Waterduck syntax):
			0678: custom_opcode_1 12 50.0 128

		Example handler prototype:
			bool __cdecl handler0678(int value1, float value1, unsigned char value3);

		Example add:
			OpcodeHookMgr->add(0x0678, 3, true, false, &handler0678);
		
		Parameters:
			opcodeNum - opcode number (0x0000 - 0x7FFF).
			paramCount - number of parameters that the opcode handles.
			hijack - should the game also handle this opcode as well as you?
			endThread - after this opcode is processed should the thread end?
			handler - pointer to the function which processes the opcode. */
	virtual bool add(short opcodeNum, int paramCount, bool hijack, bool endThread, void *handler) = 0;

	/* remove:
		Removes an opcode hook. All hooks are removed, at game shutdown, automatically.
	
		Parameters:
			opcodeNum - opcode number of the hook to remove (0x0000 - 0x7FFF). */
	virtual void remove(short opcodeNum) = 0;

	/* getScriptInfo:
		Returns a pointer to the current running script information. (Advanced use only) */
	virtual RunningScript *getScriptInfo() = 0;
};

#endif